/* sector0 - a utility to interpret the contents of sector0 of OS-9 disk */
/* written by Andrzej Kotanski, 1996, e-mail kotanski@desy.de            */

#include <stdio.h>
#include <errno.h>
#include <string.h>
#include <stdlib.h>
#include "rbf.h"

#define int4(a) ( (*a<<24) + (*(a+1)<<16) + (*(a+2)<<8) + *(a+3) )
#define int3(a) ( (*a<<16) + (*(a+1)<<8) + *(a+2) )
#define int2(a) ( (*a<<8) + *(a+1) )

struct sector0 sec0;

struct rbf_opt opt;


main(argc, argv)
char **argv;
{
    unsigned boot;
    unsigned char name[33];

    FILE *fp;
    char device[20];
    unsigned char *p;
    int i;

    if ( argc !=2 ) {
      fprintf(stderr,"%s","Usage: sector0 /device\n");
	exit(1);
    }
       
    strcpy(device, argv[1]);
/*
    strcat(device, "@");
*/    
    fp = fopen(device, "r");
    
    if ( fp == NULL ) {
	char msg[256];
       sprintf(msg,"Cannot open '%s'\n", device);
       perror(msg);
	exit(1);
	}
    if ( fread(&sec0, sizeof(struct sector0), 1, fp) == 0 ) {
       perror("Cannot read sector 0\n");
	exit(1);
	}

    memcpy(&opt, sec0.dd_opt, 32);

    printf("total sectors = %d, ", (sec0.dd_tot[0] << 16) +
          (sec0.dd_tot[1] << 8) + sec0.dd_tot[2]);
    printf("track size = %d\n", sec0.dd_tks);
    printf("bytes in alloc map = %d = $%02X, ", int2(sec0.dd_map), int2(sec0.dd_map));
    printf("cluster size = %d\n", int2(sec0.dd_bit) );
    printf("lsn of root dir = $%04X\n", int3(sec0.dd_dir));
    printf("disk owner = %d.%d\n", sec0.dd_own[0], sec0.dd_own[1]);
    printf("disk attributes = %02X, ", sec0.dd_att & 0xff);
    printf("disk id = %x\n", int2(sec0.dd_dsk) );
    printf("disk format = %x  ", sec0.dd_fmt);
    if (sec0.dd_fmt & FMT_DS)
       printf("double-sided ");
    if (sec0.dd_fmt & FMT_DD)
       printf("double-density ");
    if (sec0.dd_fmt & FMT_DT)
       printf("double-track ");
    if (sec0.dd_fmt & FMT_QT)
       printf("quad-track ");
    if (sec0.dd_fmt & FMT_OT)
       printf("octal-track ");
    printf("\n");
    printf("sectors per track = %d\n", int2(sec0.dd_spt));
    boot =  int3(sec0.dd_bt);
    if ( int2(sec0.dd_bsz) ) {
       printf("lsn of boot = $%04X\n", boot);
       printf("boot size = %d\n", int2(sec0.dd_bsz));
    } else
       printf("boot fd ptr = $%04X\n", boot);
    printf("creation date = %d/%02d/%02d %02d:%02d\n", sec0.dd_date[0]+1900,
       sec0.dd_date[1] & 0xff, sec0.dd_date[2] & 0xff, sec0.dd_date[3] & 0xff,
       sec0.dd_date[4] & 0xff);
    p = (unsigned char *) sec0.dd_name	;
    i=0;
    while ( (p[i] & 0x80) == 0 && i <32) {
		name[i]=p[i++];
    }
    name[i] = p[i] & ~0x80;
    name[i+1] = '\0';
    printf("volume name = %s\n", name);
    printf("integrity code: $%02X%02X\n", sec0.dd_sync[0],sec0.dd_sync[1]);
    printf("bitmap starting lsn = %lu\n", int4(sec0.dd_maplsn) ? int4(sec0.dd_maplsn) : 1);
	/* default if zero */
    if ( int2(sec0.dd_lsnsize) == 0 )
       sec0.dd_lsnsize[0] = 1; /* = 256 (default if zero)  */
    printf("sector size = %d\n", int2(sec0.dd_lsnsize));
    printf("sector 0 version ID = $%04X\n", int2(sec0.dd_versid));

    printf("  drive number = %d\n", opt.pd_drv);
    printf("  step rate = %d\n", opt.pd_stp);
    printf("  disk type: $%02X ", opt.pd_typ);
    if ( opt.pd_typ & TYP_HARD )
       printf("hard ");
    else
       printf("floppy ");
    if ( opt.pd_typ & TYP_DDTRK0 )
       printf("DD track 0 ");
    else
       printf("SD track 0 ");
    if ( (opt.pd_typ & TYP_SIZ_MASK) == TYP_OLD ) {
        printf(" old type floppy ");
        if ( opt.pd_typ & TYP_OLD8 )
           printf("8\" ");
	else
           printf("5.25\" ");
    }
    else {
        if ( (opt.pd_typ & TYP_SIZ_MASK) == TYP_SIZ8 )
           printf("8\" ");
        if ( (opt.pd_typ & TYP_SIZ_MASK) == TYP_SIZ5 )
           printf("5.25\" ");
        if ( (opt.pd_typ & TYP_SIZ_MASK) == TYP_SIZ3 )
           printf("3.5\" ");
    }
    printf("\n");
    printf("  density: $%02X ", opt.pd_dns);
    if ( opt.pd_dns & DNS_DD )
       printf("double  ");
    if ( opt.pd_dns & DNS_DT )
       printf("double track  ");
    if ( opt.pd_dns & DNS_QT )
       printf("quad track   ");
    if ( opt.pd_dns & DNS_OT )
       printf("octal track  ");
    printf("\n");
    printf("  cylinders(tracks) = %d\n", int2(opt.pd_cyl) );
    printf("  heads(sides) = %d\n", opt.pd_sid );
    printf("  sect/track = %d\n", int2(opt.pd_sct) );
    printf("  track 0 sectors = %d\n", int2(opt.pd_t0s) );
    printf("  verify = %s\n", opt.pd_vfy ? "on" : "off" );
    printf("  track base = %d   sector base %d\n", opt.pd_toffs, opt.pd_soffs);
    printf("  sector size = %d\n", int2(opt.pd_ssize));
    printf("  control word = $%04X\n", int2(opt.pd_cntl));

    printf("\nRBF_OPT size=%d\n", sizeof(struct rbf_opt));

}


